// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

library preconditions;

import 'package:darmatch/matchers.dart';

/// Evaluates the [precondition] against the [value] and if it doesn't match,
/// throws [PreconditionError] with description of the mismatch. Otherwise,
/// it returns the [value] unchanged. If [message] is provided, it is used
/// instead of the default exception message.
checkThat(value, Matcher precondition, [String message]) {
  bool match = precondition.matches(value);
  if (!match) {
    var exceptionMessage;
    if (message != null) {
      exceptionMessage = message;
    } else if (precondition is HasRichMessage) {
      exceptionMessage = precondition.buildMessage(value);
    } else {
      exceptionMessage = "'$value' must ${precondition.description}";
    }
    throw new PreconditionError(exceptionMessage);
  }
  return value;
}

class PreconditionError implements Error {
  final String _message;
  PreconditionError(this._message);
  String toString() => _message;
}
