// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

part of io_matchers;

class _FileExists implements Matcher<File> {
  bool matches(File file) => file.existsSync();
  String get description => "be an existing file";
}

class _FileLength implements Matcher<File> {
  Matcher<num> _matcher;

  _FileLength(this._matcher);

  bool matches(File file) {
    if (!file.existsSync()) {
      return false;
    }

    int length = file.lengthSync();
    return _matcher.matches(length);
  }

  String get description => "have length ${_matcher.description}";
}

class _FileContent implements Matcher<File> {
  Matcher<String> _matcher;

  _FileContent(this._matcher);

  bool matches(File file) {
    if (!file.existsSync()) {
      return false;
    }

    String content = file.readAsStringSync();
    return _matcher.matches(content);
  }

  String get description => "have content ${_matcher.description}";
}

/// Every existing file matches.
Matcher<File> fileExists() => new _FileExists();

/// Every existing file whose length matches given [matcher] matches.
Matcher<File> fileLength(Matcher<num> matcher) => new _FileLength(matcher);

/// Every existing file whose content (read as UTF-8 encoded text)
/// matches given [matcher] matches.
Matcher<File> fileContent(Matcher<String> matcher) => new _FileContent(matcher);
