// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

part of matchers;

class _IsInfinite implements Matcher<double> {
  bool matches(double value) => value.isInfinite;
  String get description => "be infinite";
}

class _IsNaN implements Matcher<double> {
  bool matches(double value) => value.isNaN;
  String get description => "be NaN";
}

class _ApproxEquals extends SimpleMatcher<double> {
  final double tolerance;

  _ApproxEquals(double value, this.tolerance) : super(value);

  bool matches(double value) => (value - savedValue).abs() <= tolerance;
  String get description => "be approximately equal to $savedValue";
}

/// Every double number that is infinite matches. That is,
/// the `double.INFINITY` and `double.NEGATIVE_INFINITY` constants.
Matcher<double> isInfinite() => new _IsInfinite();

/// Every double number that is NAN matches. That is, the `double.NAN`
/// constant and also all other `double`s that are NAN according to the
/// `isNaN` method.
Matcher<double> isNaN() => new _IsNaN();

/// Every double number that is approximately equal to given [value]
/// with given [tolerance] matches.
Matcher<double> approxEquals(double value, {double tolerance: 0.00001})
    => new _ApproxEquals(value, tolerance);
