// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

part of matchers;

int _maxElementLength(List/*<T>*/ list) {
  int maxElementLength = 0;
  for (final e in list) {
    int len = "$e".length;
    if (len > maxElementLength) {
      maxElementLength = len;
    }
  }
  return maxElementLength;
}

String _padLeft(String str, int length) {
  final result = new StringBuffer();
  str = str.substring(0, min(str.length, length));
  while (result.length < length - str.length) {
    result.write(' ');
  }
  result.write(str);
  return "$result";
}

String _crop(String str, int maxLength)
    => str.substring(0, min(str.length, maxLength));

String _diffLists(List/*<T>*/ expected, List/*<T>*/ actual) {
  final maxElementLengthExpected = min(30, _maxElementLength(expected));
  final maxElementLengthActual = min(30, _maxElementLength(actual));
  final commonLength = min(expected.length, actual.length);

  var result = new StringBuffer();
  for (var i = 0; i < commonLength; i++) {
    final expElem = expected[i];
    final actElem = actual[i];

    final expStr = _padLeft("$expElem", maxElementLengthExpected);
    final actStr = _crop("$actElem", maxElementLengthActual);

    result.write(expStr);
    if (expElem == actElem) {
      result.write('  ==  ');
    } else {
      result.write('  !=  ');
    }
    result.write(actStr);
    result.write('\n');
  }

  if (expected.length > actual.length) {
    final lenDiff = expected.length - actual.length;
    for (int i = 0; i < lenDiff; i++) {
      final expStr = _padLeft("${expected[i + commonLength]}", maxElementLengthExpected);
      result.write(expStr);
      result.write('  !=\n');
    }
  } else if (actual.length > expected.length) {
    final lenDiff = actual.length - expected.length;
    for (int i = 0; i < lenDiff; i++) {
      final actStr = _crop("${actual[i + commonLength]}", maxElementLengthActual);
      result.write(_padLeft("", maxElementLengthExpected));
      result.write('  !=  ');
      result.write(actStr);
      result.write('\n');
    }
  }
  return "$result";
}
