// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

part of matchers;

class _EmptyMap implements Matcher<Map> {
  bool matches(Map map) => map.isEmpty;
  String get description => "be empty";
}

class _MapContainsKey<K, V> implements Matcher<Map<K, V>> {
  K key;

  _MapContainsKey(this.key);

  bool matches(Map<K, V> map) => map.containsKey(key);
  String get description => "contain key '$key'";
}

class _MapContainsValue<K, V> implements Matcher<Map<K, V>> {
  V value;

  _MapContainsValue(this.value);

  bool matches(Map<K, V> map) => map.containsValue(value);
  String get description => "contain value '$value'";
}

class _MapContainsMapping<K, V> implements Matcher<Map<K, V>> {
  K key;
  V value;

  _MapContainsMapping(this.key, this.value);

  bool matches(Map<K, V> map) => map[key] == value;
  String get description => "contain mapping $key -> $value";
}

class _MapLength implements Matcher<Map> {
  Matcher<int> _matcher;

  _MapLength(this._matcher);

  bool matches(Map map) => _matcher.matches(map.length);
  String get description => "map length ${_matcher.description}";
}

class _MapEquals extends SimpleMatcher<Map> {
  _MapEquals(Map map) : super(map);

  bool matches(Map map) {
    if (map.length != savedValue.length) {
      return false;
    }
    var result = true;
    map.forEach((k, v) {
      if (!savedValue.containsKey(k)) {
        result = false;
      } else if (savedValue[k] != v) {
        result = false;
      }
    });
    return result;
  }
  String get description => "be equal to $savedValue";
}

/// Every map that is empty (i.e., `it.isEmpty`) matches.
Matcher<Map> emptyMap() => new _EmptyMap();

/// Every map which contains given [key] (i.e., `it.containsKey(key)`) matches.
Matcher<Map/*<K, V>*/> mapContainsKey(/*K*/ key) => new _MapContainsKey/*<K, V>*/(key);

/// Every map which contains given [value] (i.e., `it.containsValue(value)`) matches.
Matcher<Map/*<K, V>*/> mapContainsValue(/*V*/ value) => new _MapContainsValue/*<K, V>*/(value);

/// Every map which contains mapping from given [key] to given [value]
/// (i.e., `it[key] == value`) matches.
Matcher<Map/*<K, V>*/> mapContains(/*K*/ key, /*V*/ value) => new _MapContainsMapping/*<K, V>*/(key, value);

/// Every map whose length matches given [matcher] matches.
Matcher<Map> mapLength(Matcher<int> matcher) => new _MapLength(matcher);

/// Every map that contains the same mappings as given [map] matches.
/// Iteration order is _not_ significant.
Matcher<Map> mapEquals(Map map) => new _MapEquals(map);
