// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

part of matchers;

class _Zero implements Matcher<num> {
  bool matches(num value) => value == 0;
  String get description => "be zero";
}

class _Positive implements Matcher<num> {
  bool matches(num value) => value > 0;
  String get description => "be positive";
}

class _Negative implements Matcher<num> {
  bool matches(num value) => value < 0;
  String get description => "be negative";
}

class _LessThan extends SimpleMatcher<num> {
  _LessThan(num value) : super(value);

  bool matches(num value) => value < savedValue;
  String get description => "be less than $savedValue";
}

class _LessThanOrEqual extends SimpleMatcher<num> {
  _LessThanOrEqual(num value) : super(value);

  bool matches(num value) => value <= savedValue;
  String get description => "be less than or equal to $savedValue";
}

class _GreaterThan extends SimpleMatcher<num> {
  _GreaterThan(num value) : super(value);

  bool matches(num value) => value > savedValue;
  String get description => "be greater than $savedValue";
}

class _GreaterThanOrEqual extends SimpleMatcher<num> {
  _GreaterThanOrEqual(num value) : super(value);

  bool matches(num value) => value >= savedValue;
  String get description => "be greater than or equal to $savedValue";
}

class _InRange implements Matcher<num> {
  num _low, _high;

  _InRange(num this._low, num this._high);

  bool matches(num value) => value >= _low && value <= _high;
  String get description => "be in range from $_low to $_high";
}

/// Every number that is equal to zero (i.e., `it == 0`) matches.
Matcher<num> zero() => new _Zero();

/// Every number that is positive (i.e., `it > 0`) matches.
Matcher<num> positive() => new _Positive();

/// Every number that is negative (i.e., `it < 0`) matches.
Matcher<num> negative() => new _Negative();

/// Every number that is smaller than given [value] (i.e., `it < value`) matches.
Matcher<num> lessThan(num value) => new _LessThan(value);

/// Every number that is smaller than or equal to given [value]
/// (i.e., `it <= value`) matches.
Matcher<num> lessThanOrEqual(num value) => new _LessThanOrEqual(value);

/// Every number that is greater than given [value] (i.e., `it > value`) matches.
Matcher<num> greaterThan(num value) => new _GreaterThan(value);

/// Every number that is greater than or equal to given [value]
/// (i.e., `it >= value`) matches.
Matcher<num> greaterThanOrEqual(num value) => new _GreaterThanOrEqual(value);

/// Every number that is in given range (i.e., `it >= from && it <= to`) matches.
Matcher<num> inRange(num from, num to) => new _InRange(from, to);
