// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

import 'package:darmatch/matchers.dart';
import 'package:darmatch/preconditions.dart';

import 'package:detester/console.dart';
part 'package:detester/console_test.dart';

suite() => [
  testPreconditions,
  testPreconditionMessage
];

class TinyClass {
  int positiveNum;

  TinyClass(int positiveNum) {
    this.positiveNum = checkThat(positiveNum, positive(), 'must be positive');
  }

  int addToNegativeNum(int negativeNum) {
    checkThat(negativeNum, negative());
    return positiveNum + negativeNum;
  }
}

testPreconditions() {
  expectThat(() => new TinyClass(1), returnsNormally());
  expectThat(() => new TinyClass(0), throws(new IsType<PreconditionError>()));
  expectThat(() => new TinyClass(-1), throws(new IsType<PreconditionError>()));

  var t = new TinyClass(1);
  expectThat(() => t.addToNegativeNum(1), throws());
  expectThat(() => t.addToNegativeNum(0), throws());
  expectThat(() => t.addToNegativeNum(-1), returnsNormally());

  expectThat(t.addToNegativeNum(-1), equals(0));
}

testPreconditionMessage() {
  // user-defined message
  expectThat(() => new TinyClass(-1), throws(stringForm(equals('must be positive'))));

  // default message
  var t = new TinyClass(1);
  expectThat(() => t.addToNegativeNum(1), throws(stringForm(equals("'1' must be negative"))));
}

